#include "stdafx.h"
#include "FormatSize.h"
#include "locality.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/**
 * @name Constants for short sizes.
 */
/* @{ */
static const UINT KILO = 1024;
static const UINT MEGA = 1024 * KILO;
static const UINT GIGA = 1024 * MEGA;
static const __int64 TERA = 1024 * (__int64) GIGA;

static const char IDS_SUFFIX_BYTE[]   =      " B";
static const char IDS_SUFFIX_KILO[]   =      " KB";
static const char IDS_SUFFIX_MEGA[]   =      " MB";
static const char IDS_SUFFIX_GIGA[]   =      " GB";
static const char IDS_SUFFIX_TERA[]   =      " TB";
static const char IDS_SUFFIX_PETA[]   =      " PB";

/**
 * @}
 */


/**
 * @brief Formats a size as a long string
 *
 * MakeShortSize(500) = "500"
 * MakeShortSize(1024) = "1,024"
 * MakeShortSize(12000) = "12,000"
 * MakeShortSize(200000) = "200,000"
 * @param [in] size File's size to convert.
 * @return Size string with inserted commas
 * @todo Can't handle > terabyte filesizes.
 */
CString MakeLongSize(__int64 size)
{
	CString str;
	str.Format("%I64d", size);
	str = locality::GetLocaleStr(str, 0);
	return str;
}

/**
 * @brief Formats a size as a short string.
 *
 * MakeShortSize(500) = "500b"
 * MakeShortSize(1024) = "1Kb"
 * MakeShortSize(12000) = "1.7Kb"
 * MakeShortSize(200000) = "195Kb"
 * @param [in] size File's size to convert.
 * @return Size string with localized suffix.
 * @note Localized suffix strings are read from resource.
 * @todo Can't handle > terabyte filesizes.
 */
CString MakeShortSize(__int64 size)
{
#pragma warning(disable:4244) // warning C4244: '=' : conversion from '__int64' to 'double', possible loss of data
	double fsize = size;
#pragma warning(default:4244) // warning C4244: '=' : conversion from '__int64' to 'double', possible loss of data
	double number = 0;
	int ndigits = 0;
	CString suffix;

	if (size < KILO)
	{
		number = fsize;
		suffix = IDS_SUFFIX_BYTE; // VERIFY(suffix.LoadString(IDS_SUFFIX_BYTE));
	}
	else if (size < MEGA)
	{
		number = fsize / KILO;
		suffix = IDS_SUFFIX_KILO; // VERIFY(suffix.LoadString(IDS_SUFFIX_KILO));
		if (size < KILO * 10)
		{
			ndigits = 2;
		}
		else if (size < KILO * 100)
		{
			ndigits = 1;
		}
	}
	else if (size < GIGA)
	{
		number = fsize / (MEGA);
		suffix = IDS_SUFFIX_MEGA; // VERIFY(suffix.LoadString(IDS_SUFFIX_MEGA));
		if (size < MEGA * 10)
		{
			ndigits = 2;
		}
		else if (size < MEGA * 100)
		{
			ndigits = 1;
		}
	}
	else if (size < (__int64)TERA)
	{
		number = fsize / ((__int64)GIGA);
		suffix = IDS_SUFFIX_GIGA; // VERIFY(suffix.LoadString(IDS_SUFFIX_GIGA));
		if (size < (__int64)GIGA * 10)
		{
			ndigits = 2;
		}
		else if (size < (__int64)GIGA * 100)
		{
			ndigits = 1;
		}
	}
	else
	{
		// overflow (?) -- show ">TB"
		CString s(_T(">"));
		suffix = IDS_SUFFIX_TERA; // VERIFY(suffix.LoadString(IDS_SUFFIX_TERA));
		s += suffix;
		return s;
	}

	CString s;
	s.Format(_T("%lf"), number);
	s = locality::GetLocaleStr(s, ndigits) + suffix;
	return s;
}

